"""Command Line Interface for synchronising Google Calendar and Notion."""
import asyncio
import datetime
import logging
from pathlib import Path
from typing import Callable, Coroutine, Optional

import arrow
import notion_client as nc
import typer
from googleapiclient.discovery import Resource  # type: ignore
from pydantic import ValidationError

from ncal import core
from ncal.config import Settings, load_settings
from ncal.gcal_token import gcal_token

from . import __version__

app = typer.Typer(help="CLI to sync a Notion database with Google Calendar.")
state = {"verbose": False}


async def scheduler(
    timedelta: datetime.timedelta, function: Callable[..., Coroutine], f_args: dict
) -> None:
    """Schedule an async function.

    Args:
        timedelta:
        function:
        f_args: arguments to pass to the function
    """
    next_run = arrow.utcnow() + timedelta
    print(f"next run: {next_run}")
    while True:
        now = arrow.utcnow()
        if now >= next_run:
            await function(**f_args)
            next_run = timedelta + next_run
            print(f"next run: {next_run}")
        await asyncio.sleep(1)


async def sync(settings: Settings, service: Resource, notion: nc.Client) -> None:
    """Sync between Google Calendar and Notion.

    Args:
        settings: Configuration settings
        service: A Google Calendar API Client
        notion: A Notion API Client
    """
    num_steps = 4
    if settings.delete_option:
        num_steps += 1

    with typer.progressbar(
        range(num_steps), label="Synchronising", show_eta=False, show_pos=True
    ) as progress:
        today_date = arrow.utcnow().isoformat()

        progress.label = "new N->G"
        core.new_events_notion_to_gcal(
            settings.database_id,
            settings.url_root,
            settings.default_calendar_name,
            settings.calendar_dictionary,
            settings.task_notion_name,
            settings.date_notion_name,
            settings.initiative_notion_name,
            settings.extrainfo_notion_name,
            settings.on_gcal_notion_name,
            settings.gcal_event_id_notion_name,
            settings.lastupdatedtime_notion_name,
            settings.calendar_notion_name,
            settings.current_calendar_id_notion_name,
            settings.delete_notion_name,
            notion,
            service,
            settings=settings,
        )
        progress.update(1)

        progress.label = "modified N->G"
        core.existing_events_notion_to_gcal(
            settings.database_id,
            settings.url_root,
            settings.default_calendar_id,
            settings.default_calendar_name,
            settings.calendar_dictionary,
            settings.task_notion_name,
            settings.date_notion_name,
            settings.initiative_notion_name,
            settings.extrainfo_notion_name,
            settings.on_gcal_notion_name,
            settings.need_gcal_update_notion_name,
            settings.gcal_event_id_notion_name,
            settings.lastupdatedtime_notion_name,
            settings.calendar_notion_name,
            settings.current_calendar_id_notion_name,
            settings.delete_notion_name,
            notion,
            today_date,
            service,
            settings=settings,
        )
        progress.update(1)

        progress.label = "modified G->N"
        core.existing_events_gcal_to_notion(
            settings.database_id,
            settings.default_calendar_name,
            settings.calendar_dictionary,
            settings.date_notion_name,
            settings.on_gcal_notion_name,
            settings.need_gcal_update_notion_name,
            settings.gcal_event_id_notion_name,
            settings.lastupdatedtime_notion_name,
            settings.calendar_notion_name,
            settings.current_calendar_id_notion_name,
            settings.delete_notion_name,
            service,
            notion,
            today_date,
            settings=settings,
        )
        progress.update(1)

        progress.label = "new G->N"
        core.new_events_gcal_to_notion(
            settings.database_id,
            settings.calendar_dictionary,
            settings.task_notion_name,
            settings.date_notion_name,
            settings.extrainfo_notion_name,
            settings.on_gcal_notion_name,
            settings.gcal_event_id_notion_name,
            settings.lastupdatedtime_notion_name,
            settings.calendar_notion_name,
            settings.current_calendar_id_notion_name,
            settings.delete_notion_name,
            service,
            notion,
            settings=settings,
        )

        if settings.delete_option:
            progress.update(1)
            progress.label = "delete done pages from GCal"
            core.delete_done_pages(
                notion=notion,
                database_id=settings.database_id,
                GCalEventId_Notion_Name=settings.gcal_event_id_notion_name,
                On_GCal_Notion_Name=settings.on_gcal_notion_name,
                Delete_Notion_Name=settings.delete_notion_name,
                DELETE_OPTION=settings.delete_option,
                calendarDictionary=settings.calendar_dictionary,
                Calendar_Notion_Name=settings.calendar_notion_name,
                service=service,
            )

        progress.label = "Synchronised"
        progress.update(1)
        typer.echo(f"Synchronised at UTC {arrow.utcnow()}")


async def continuous_sync(
    interval: datetime.timedelta,
    settings: Settings,
    service: Resource,
    notion: nc.Client,
):
    """Call sync continuously.

    Args:
        interval (datetime.timedelta):
        settings (Settings):
        service (Resource):
        notion (nc.Client):
    """
    await sync(settings, service, notion)
    await scheduler(
        interval, sync, {"settings": settings, "service": service, "notion": notion}
    )


@app.command("sync")
def cli_sync(
    repeat: bool = typer.Option(False, "--repeat/--no-repeat", "-r"),
    seconds: int = 10,
    config_file: Optional[Path] = typer.Option(
        None, "--config-file", "-c", help="toml configuration file location"
    ),
    notion_api_token: Optional[str] = None,
    database_id: Optional[str] = None,
    url_root: Optional[str] = None,
    delete_pages: Optional[bool] = typer.Option(
        False,
        "--delete-pages/--no-delete-pages",
        "-d",
        help="delete pages which have been marked done",
    ),
):
    """CLI to sync a Notion database with Google Calendar."""
    typer.echo()
    typer.secho(
        "Synchronize Notion <-> GCal", bg=typer.colors.GREEN, fg="white", bold=True
    )
    typer.echo()

    try:
        if config_file:
            settings = load_settings(
                config_file_path=config_file,
                notion_api_token=notion_api_token,
                database_id=database_id,
                url_root=url_root,
                delete_option=delete_pages,
            )
        else:
            settings = load_settings(
                notion_api_token=notion_api_token,
                database_id=database_id,
                url_root=url_root,
                delete_option=delete_pages,
            )
    except ValidationError:
        typer.secho(
            "no valid config provided, use --help for more information",
            bg="white",
            fg="red",
        )
        typer.echo("Required options: notion_api_token, database_id, urlRoot")
        raise typer.Exit(1)
    except FileNotFoundError:
        typer.secho(
            "invalid config-file provided, use --help for more information",
            bg="white",
            fg="red",
        )
        raise typer.Exit(1)
    logging.info(settings)

    typer.echo("Setting up API connections...")
    service, calendar, notion = core.setup_api_connections(
        default_calendar_id=settings.default_calendar_id,
        credentials_location=settings.credentials_location,
        notion_api_token=settings.notion_api_token,
    )

    if repeat:
        interval = datetime.timedelta(seconds=seconds)
        asyncio.run(continuous_sync(interval, settings, service, notion))
    else:
        asyncio.run(sync(settings, service, notion))


@app.command("gcal-token")
def cli_gcal_token(client_secret_json: Path, out_file: Path = Path("token.pkl")):
    """Generate a token, which will be stored in a .pkl file."""
    gcal_token(out_file, client_secret_json)


@app.callback(invoke_without_command=True, no_args_is_help=True)
def main(
    verbose: bool = False,
    version: bool = typer.Option(None, "--version", is_eager=True),
):
    """Main CLI entrypoint."""  # noqa: D401
    if verbose:
        state["verbose"] = True
        logging.basicConfig(level=20)
    if version:
        typer.echo(f"ncal version: {__version__}")
        raise typer.Exit()



"""Some utility functions for working with Notion properties etc."""
from typing import Any, Literal

import notion_client


def collapse_rich_text_property(property: list[dict[str, Any]]) -> str:
    """Collapse a Notion Rich Text property into a single string.

    Args:
        property: An array of rich text objects (as returned by a rich text property)
    Returns:
        The plain_text of the rich text objects concatenated.
    """
    return "".join((i["plain_text"] for i in property))


def get_relation_title(
    notion: notion_client.Client, notion_page: dict[str, Any], relation_name: str
) -> str:
    """Get the title of the first page in a relation property."""
    relation_property: list = notion_page["properties"][relation_name]["relation"]
    if relation_property:
        relation_id: str = relation_property[0]["id"]
        relation_title: dict = notion.pages.properties.retrieve(
            relation_id, "title"
        )  # type:ignore
        return relation_title["results"][0]["title"]["plain_text"]
    else:
        return ""


def get_property_text(
    notion: notion_client.Client,
    notion_page: dict[str, Any],
    property_name: str,
    property_type: Literal["relation", "select"],
) -> str:
    """Get the text contained within several different types of property.

    Note: Currently only relation and select are implemented.
    """
    text: str
    if property_type == "select":
        text = notion_page["properties"][property_name]["select"]["name"]
    elif property_type == "relation":
        text = get_relation_title(
            notion=notion, notion_page=notion_page, relation_name=property_name
        )
    else:
        raise ValueError
    return text



"""Produce an access token for the Google Calendar API."""
import pickle

from google_auth_oauthlib.flow import InstalledAppFlow  # type: ignore


def gcal_token(credentials_location, client_secret_file):
    """Set up the Google Calendar api interface."""
    scopes = ["https://www.googleapis.com/auth/calendar"]
    flow = InstalledAppFlow.from_client_secrets_file(client_secret_file, scopes=scopes)
    credentials = flow.run_console()
    pickle.dump(credentials, open(credentials_location, "wb"))



"""
Functions and classes to manage configuration are in this file.

Configuration can operate through toml, .env, environment variables, or the command
line.
"""
from os import environ
from pathlib import Path
from typing import Any, Dict, Literal, Optional

import pydantic
import tomli
from dotenv import load_dotenv


class Settings(pydantic.BaseModel):
    """Class for storing settings.

    Attributes:
        notion_api_token: API token for Notion integration
        credentials_location: file containing GCal Credentials
        database_id: ID of the Notion database
    """

    notion_api_token: str

    # get the mess of numbers before the "?" on your dashboard URL (no need to split
    # into dashes)
    database_id: str
    # open up a task and then copy the URL root up to the "p="
    url_root: str
    # GCalToken creating program TODO:this might not be necessary anymore?!
    run_script: str = "python3 GCalToken.py"
    # Pickle file containing GCal Credentials
    credentials_location: Path = Path("token.pkl")

    default_event_length: int = 60  # Default event length in minutes
    # http://www.timezoneconverter.com/cgi-bin/zonehelp.tzc  TODO: make this unnecessary
    timezone: str = "Europe/London"

    # 8 would be 8 am. 16 would be 4 pm. Only whole numbers
    default_event_start: int = 8

    # 0 if you want dates on your Notion dashboard to be treated as an all-day event
    all_day_event_option: int = 0

    # ^^ 1 if you want dates on your Notion dashboard to be created at whatever hour you
    # defined in the DEFAULT_EVENT_START variable

    # MULTIPLE CALENDAR PART:
    #  - VERY IMPORTANT: For each 'key' of the dictionary, make sure that you make that
    # EXACT thing in the Notion database first before running the code. You WILL have
    # an error and your dashboard/calendar will be messed up

    # The GCal calendar id. The format is something like
    # "sldkjfliksedjgodsfhgshglsj@group.calendar.google.com"
    default_calendar_id: str = "cv9aoe96819t4bt5f5jars9b10@group.calendar.google.com"

    default_calendar_name: str = "Notion Events"

    # leave the first entry as is
    # the structure should be as follows:
    # WHAT_THE_OPTION_IN_NOTION_IS_CALLED: GCAL_CALENDAR_ID
    calendar_dictionary = {
        default_calendar_name: default_calendar_id,
        # "Cal Name": "asdfadsf234asef21134@group.calendar.google.com",
    }

    # doesn't delete the Notion task (yet), I'm waiting for the Python API to be
    # updated to allow deleting tasks
    delete_option: bool = False
    # set at True if you want the delete column being checked off to mean that the
    # gCal event and the Notion Event will be checked off.
    # set at False if you want nothing deleted

    # DATABASE SPECIFIC EDITS
    # There needs to be a few properties on the Notion Database for this to work.
    # Replace the values of each variable with the string of what the variable is
    # called on your Notion dashboard
    # The Last Edited Time column is a property of the notion pages themselves,
    # you just have to make it a column
    # The NeedGCalUpdate column is a formula column that works as such
    # "if(prop("Last Edited Time") > prop("Last Updated Time"), true, false)"
    # Please refer to the Template if you are confused:
    # https://www.notion.so/akarri/2583098dfd32472ab6ca1ff2a8b2866d?v=3a1adf60f15748f08ed925a2eca88421

    task_notion_name: str = "Name"
    date_notion_name: str = "Do Date"
    initiative_notion_name: str = "Project"
    initiative_notion_type: Literal["relation", "select"] = "relation"
    extrainfo_notion_name: str = "Description"
    on_gcal_notion_name: str = "On GCal?"
    need_gcal_update_notion_name: str = "NeedGCalUpdate"
    gcal_event_id_notion_name: str = "GCal Event Id"
    lastupdatedtime_notion_name: str = "Last Updated Time"
    calendar_notion_name: str = "Calendar"
    current_calendar_id_notion_name: str = "Current Calendar Id"
    delete_notion_name: str = "Done"


def load_config_file(path_to_file: Path) -> Dict[str, Any]:
    """Load a toml config file into a dictionary.

    Args:
        path_to_file:
    Returns:
        Settings (unfiltered)
    """
    with open(path_to_file, "rb") as f:
        tomli_dictionary = tomli.load(f)
        return tomli_dictionary


def env_var_names_dict(prefix: str) -> Dict[str, str]:
    """
    Args:
        prefix: string to add to the start of each environment variable name
    Returns:
        Dict[str, str]: A dictionary in the form of `original_name: new_name`

    ```python
    >>> env_var_names_dict("PREFIX_")
    {'notion_api_token': 'prefix_notion_api_token', 'database_id': 'prefix_database_id', 'url_root': 'prefix_url_root', 'run_script': 'prefix_run_script', 'credentials_location': 'prefix_credentials_location', 'default_event_length': 'prefix_default_event_length', 'timezone': 'prefix_timezone', 'default_event_start': 'prefix_default_event_start', 'all_day_event_option': 'prefix_all_day_event_option', 'default_calendar_id': 'prefix_default_calendar_id', 'default_calendar_name': 'prefix_default_calendar_name', 'delete_option': 'prefix_delete_option', 'task_notion_name': 'prefix_task_notion_name', 'date_notion_name': 'prefix_date_notion_name', 'initiative_notion_name': 'prefix_initiative_notion_name', 'initiative_notion_type': 'prefix_initiative_notion_type', 'extrainfo_notion_name': 'prefix_extrainfo_notion_name', 'on_gcal_notion_name': 'prefix_on_gcal_notion_name', 'need_gcal_update_notion_name': 'prefix_need_gcal_update_notion_name', 'gcal_event_id_notion_name': 'prefix_gcal_event_id_notion_name', 'lastupdatedtime_notion_name': 'prefix_lastupdatedtime_notion_name', 'calendar_notion_name': 'prefix_calendar_notion_name', 'current_calendar_id_notion_name': 'prefix_current_calendar_id_notion_name', 'delete_notion_name': 'prefix_delete_notion_name', 'calendar_dictionary': 'prefix_calendar_dictionary'}

    ```
    """  # noqa
    return {i: prefix.lower() + i.lower() for i in Settings.__fields__.keys()}


def get_env_vars_case_insensitive(env_var_names: Dict[str, str]) -> Dict[str, str]:
    """Get environment variables matching the given dictionary.

    Args:
        env_var_names: {desired label: environment variable label}
    Returns:
        Dictionary of environment variables
    """
    load_dotenv()  # uses .env file if available
    ncal_env_vars: dict[str, str] = {}
    # populate env_vars with values which match the dictionary
    for key, value in env_var_names.items():
        if (x := value.upper()) in environ:
            ncal_env_vars[key] = environ[x]
        if (x := value.lower()) in environ:
            ncal_env_vars[key] = environ[x]
    return ncal_env_vars


def load_settings(
    config_file_path: Optional[Path] = None,
    *,
    use_env_vars: bool = True,
    use_toml_file: bool = True,
    **kwargs: Any,
) -> Settings:
    """Load settings from multiple sources.

    This function falls back to defaults, so only a few settings are required.
    Priority (high-low): cli, env, config_file.

    Args:
        config_file_path: Path to a .toml config file
        use_env_vars:
        use_toml_file:
        **kwargs: Other settings to provide to the Settings object

    Returns:
        A ncal.config.Settings object
    """
    default_settings = Settings.construct().dict()  # type: ignore

    if use_env_vars:
        prefix = "NCAL_"
        env_var_names = {i: prefix + i for i in Settings.__fields__.keys()}
        env_settings = get_env_vars_case_insensitive(env_var_names)
    else:
        env_settings = {}

    if config_file_path is not None:
        try:
            toml_settings = load_config_file(config_file_path)
        except FileNotFoundError:
            print(f"no config file found at {config_file_path}")
            toml_settings = {}
    else:
        toml_settings = {}

    passed_settings = {key: value for key, value in kwargs.items() if value is not None}

    settings = {**default_settings, **toml_settings, **env_settings, **passed_settings}
    return Settings(**settings)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ncal']

package_data = \
{'': ['*']}

install_requires = \
['arrow>=1.2.1,<2.0.0',
 'google-api-python-client>=2.36.0,<3.0.0',
 'google-auth-oauthlib>=0.4.6,<0.6.0',
 'notion-client>=0.9.0,<0.10.0',
 'pydantic>=1.9.0,<2.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'python-dotenv>=0.19.2,<0.20.0',
 'tomli>=2.0.0,<3.0.0',
 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['ncal = ncal.cli:app']}

setup_kwargs = {
    'name': 'ncal',
    'version': '0.1.15',
    'description': '',
    'long_description': '[![CodeQL](https://github.com/SG60/ncal/actions/workflows/codeql-analysis.yml/badge.svg)](https://github.com/SG60/ncal/actions/workflows/codeql-analysis.yml)\n[![Package Tests](https://github.com/SG60/ncal/actions/workflows/tests.yml/badge.svg)](https://github.com/SG60/ncal/actions/workflows/tests.yml)\n[![codecov](https://codecov.io/gh/SG60/ncal/branch/main/graph/badge.svg?token=UZCOEA0YWQ)](https://codecov.io/gh/SG60/ncal)\n[![Code Style](https://github.com/SG60/ncal/actions/workflows/code-style.yml/badge.svg)](https://github.com/SG60/ncal/actions/workflows/code-style.yml)\n[![Lines of Code](https://sonarcloud.io/api/project_badges/measure?project=SG60_ncal&metric=ncloc)](https://sonarcloud.io/summary/new_code?id=SG60_ncal)\n  \n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/ncal?label=supported%20python)](https://pypi.org/project/ncal/)\n[![PyPI](https://img.shields.io/pypi/v/ncal?logo=python)](https://pypi.org/project/ncal/)\n[![Docker Image Version (latest semver)](https://img.shields.io/docker/v/sg60/ncal?label=docker&logo=docker)](https://hub.docker.com/r/sg60/ncal)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n# 2-Way Sync for Notion and Google Calendar\n  \nCurrently reworking the documentation.\n\nInstall with `pip install ncal`. This will install the `ncal` command.\n\n## Configuration\nConfiguration is via toml, command line flags, or environment variables (including via a .env file).\n\nConfiguration is via toml, command line flags, or environment variables (including via a .env file). Reading through `config.py` will give a lot of useful information on options. Run `ncal --help` to get more info on the cli command.\n\nSee [old README.md](https://github.com/SG60/ncal/blob/main/oldREADME.md) for more info.\n',
    'author': 'Sam Greening',
    'author_email': 'samjg60@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://sg60.github.io/ncal/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""Core functionality for synchronisation."""
import datetime
import logging
import pickle
import time
from typing import Any

import arrow
import dateutil.parser
import google.auth.exceptions  # type: ignore
import googleapiclient.discovery  # type: ignore
import notion_client as nc  # type: ignore
from googleapiclient.errors import HttpError  # type: ignore

from ncal import config, notion_utils
from ncal.gcal_token import gcal_token
from ncal.notion_utils import get_property_text


def setup_google_api(
    calendar_id: str, credentials_location: str
) -> tuple[googleapiclient.discovery.Resource, Any]:
    """Set up the Google Calendar API interface."""
    credentials = pickle.load(open(credentials_location, "rb"))
    service = googleapiclient.discovery.build("calendar", "v3", credentials=credentials)

    # There could be a hiccup if the Google Calendar API token expires.
    # If the token expires, we create a new token for the program to use
    try:
        calendar = service.calendars().get(calendarId=calendar_id).execute()
    except google.auth.exceptions.RefreshError:
        # refresh the token

        gcal_token(credentials_location, "client_secret.json")

        # SET UP THE GOOGLE CALENDAR API INTERFACE

        credentials = pickle.load(open(credentials_location, "rb"))
        service = googleapiclient.discovery.build(
            "calendar", "v3", credentials=credentials
        )

        calendar = service.calendars().get(calendarId=calendar_id).execute()

    return (service, calendar)


def setup_api_connections(
    default_calendar_id: str,
    credentials_location,
    notion_api_token: str,
) -> tuple[googleapiclient.discovery.Resource, Any, nc.Client]:
    """Setup the API connections to Google Calendar and notion
    Args:
        default_calendar_id: gcal calendar Id
        credentials_location: location of the credentials pickle file
        notion_api_token: token from the notion api
    Returns:
        (google api service, calendar, notion client)
    """
    # setup google api
    service, calendar = setup_google_api(
        default_calendar_id,
        str(credentials_location),
    )
    # This is where we set up the connection with the Notion API
    notion = nc.Client(auth=notion_api_token)
    return service, calendar, notion


def paginated_database_query(
    notion_client: nc.Client, database_id: str, **query: Any
) -> list:
    """similar to notion_client.database.query(**query)
    Args:
        notion_client:
        database_id:
        **query: A query such as would be used for the normal notion_client query
    Returns:
        List of notion pages matching the query
    """
    matching_pages = []

    while True:
        # this query will return a dictionary that we will parse
        # for information that we want
        response = notion_client.databases.query(database_id, **query)
        matching_pages.extend(response["results"])  # type: ignore
        if response["next_cursor"]:  # type: ignore
            query["start_cursor"] = response["next_cursor"]  # type: ignore
        else:
            break
    return matching_pages


def new_events_notion_to_gcal(
    database_id,
    url_root,
    DEFAULT_CALENDAR_NAME,
    calendarDictionary,
    Task_Notion_Name,
    Date_Notion_Name,
    Initiative_Notion_Name,
    ExtraInfo_Notion_Name,
    On_GCal_Notion_Name,
    GCalEventId_Notion_Name,
    LastUpdatedTime_Notion_Name,
    Calendar_Notion_Name,
    Current_Calendar_Id_Notion_Name,
    Delete_Notion_Name,
    notion,
    service,
    settings: config.Settings,
):
    """
    Take Notion Events not on GCal and move them over to GCal.

    If you just want all Notion events to be on GCal, then you'll have to edit the
    query so it is only checking the 'On GCal?' property
    """

    def get_new_notion_pages(
        database_id: str,
        on_gcal_notion_name: str,
        date_notion_name: str,
        delete_notion_name: str,
        notion: nc.Client,
    ) -> list:
        """Get new pages from notion (with pagination!)."""
        matching_pages = []

        query = {
            # "database_id": database_id,
            "filter": {
                "and": [
                    {
                        "property": on_gcal_notion_name,
                        "checkbox": {"equals": False},
                    },
                    {"property": delete_notion_name, "checkbox": {"equals": False}},
                ]
            },
        }

        matching_pages = paginated_database_query(notion, database_id, **query)
        return matching_pages

    resultList = get_new_notion_pages(
        database_id, On_GCal_Notion_Name, Date_Notion_Name, Delete_Notion_Name, notion
    )

    # logging.info(len(resultList))

    try:
        logging.info(resultList[0])
    except IndexError:
        logging.info("index error")

    task_names = []
    start_dates = []
    end_times = []
    initiatives = []
    extra_info = []
    url_list = []
    cal_event_id_list = []
    calendar_list = []

    if len(resultList) > 0:
        for i, el in enumerate(resultList):
            logging.info(el)

            task_names.append(
                notion_utils.collapse_rich_text_property(
                    el["properties"][Task_Notion_Name]["title"]
                )
            )
            start_dates.append(el["properties"][Date_Notion_Name]["date"]["start"])

            if el["properties"][Date_Notion_Name]["date"]["end"] is not None:
                end_times.append(el["properties"][Date_Notion_Name]["date"]["end"])
            else:
                end_times.append(el["properties"][Date_Notion_Name]["date"]["start"])

            try:
                initiatives.append(
                    get_property_text(
                        notion=notion,
                        notion_page=el,
                        property_name=Initiative_Notion_Name,
                        property_type=settings.initiative_notion_type,
                    )
                )
            except ValueError:
                initiatives.append("")

            try:
                extra_info.append(
                    el["properties"][ExtraInfo_Notion_Name]["rich_text"][0]["text"][
                        "content"
                    ]
                )
            except IndexError:
                extra_info.append("")
            url_list.append(make_task_url(el["id"], url_root))

            try:
                calendar_list.append(
                    calendarDictionary[
                        el["properties"][Calendar_Notion_Name]["select"]["name"]
                    ]
                )
            # keyerror occurs when there's nothing put into the calendar in the first
            # place
            except (KeyError, TypeError):
                calendar_list.append(calendarDictionary[DEFAULT_CALENDAR_NAME])

            pageId = el["id"]
            # This checks off that the event has been put on Google Calendar
            notion.pages.update(
                **{
                    "page_id": pageId,
                    "properties": {
                        On_GCal_Notion_Name: {"checkbox": True},
                        LastUpdatedTime_Notion_Name: {
                            "date": {
                                "start": arrow.utcnow().isoformat(),
                                "end": None,
                            }
                        },
                    },
                },
            )
            logging.info(calendar_list)

            def create_gcal_event(
                task_name,
                initiative,
                extra_info,
                start,
                end,
                url,
                calendar,
                service,
                settings: config.Settings,
            ):
                # 2 Cases: Start and End are  both either date or date+time
                # Have restriction that the calendar events don't cross days
                try:
                    # start and end are both dates
                    cal_event_id = make_cal_event(
                        task_name,
                        make_event_description(initiative, extra_info),
                        datetime.datetime.strptime(start, "%Y-%m-%d"),
                        url,
                        datetime.datetime.strptime(end, "%Y-%m-%d"),
                        calendar,
                        service,
                        settings,
                    )
                except ValueError:
                    try:
                        # start and end are both date+time
                        cal_event_id = make_cal_event(
                            task_name,
                            make_event_description(initiative, extra_info),
                            dateutil.parser.isoparse(start),
                            url,
                            dateutil.parser.isoparse(end),
                            calendar,
                            service,
                            settings,
                        )
                    except ValueError:
                        cal_event_id = make_cal_event(
                            task_name,
                            make_event_description(initiative, extra_info),
                            dateutil.parser.isoparse(start),
                            url,
                            dateutil.parser.isoparse(end),
                            calendar,
                            service,
                            settings,
                        )
                return cal_event_id

            calEventId = create_gcal_event(
                task_names[i],
                initiatives[i],
                extra_info[i],
                start_dates[i],
                end_times[i],
                url_list[i],
                calendar_list[i],
                service,
                settings,
            )
            cal_event_id_list.append(calEventId)

            if (
                calendar_list[i] == calendarDictionary[DEFAULT_CALENDAR_NAME]
            ):  # this means that there is no calendar assigned on Notion
                # This puts the the GCal Id into the Notion Dashboard
                notion.pages.update(
                    **{
                        "page_id": pageId,
                        "properties": {
                            GCalEventId_Notion_Name: {
                                "rich_text": [
                                    {"text": {"content": cal_event_id_list[i]}}
                                ]
                            },
                            Current_Calendar_Id_Notion_Name: {
                                "rich_text": [{"text": {"content": calendar_list[i]}}]
                            },
                            Calendar_Notion_Name: {
                                "select": {"name": DEFAULT_CALENDAR_NAME},
                            },
                        },
                    },
                )
            else:  # just a regular update
                notion.pages.update(
                    **{
                        "page_id": pageId,
                        "properties": {
                            GCalEventId_Notion_Name: {
                                "rich_text": [
                                    {"text": {"content": cal_event_id_list[i]}}
                                ]
                            },
                            Current_Calendar_Id_Notion_Name: {
                                "rich_text": [{"text": {"content": calendar_list[i]}}]
                            },
                        },
                    },
                )

    else:
        logging.info("Nothing new added to GCal")
    return


def existing_events_notion_to_gcal(
    database_id,
    url_root,
    DEFAULT_CALENDAR_ID,
    DEFAULT_CALENDAR_NAME,
    calendarDictionary,
    Task_Notion_Name,
    Date_Notion_Name,
    Initiative_Notion_Name,
    ExtraInfo_Notion_Name,
    On_GCal_Notion_Name,
    NeedGCalUpdate_Notion_Name,
    GCalEventId_Notion_Name,
    LastUpdatedTime_Notion_Name,
    Calendar_Notion_Name,
    Current_Calendar_Id_Notion_Name,
    Delete_Notion_Name,
    notion,
    todayDate,
    service,
    settings: config.Settings,
):
    """
    Updating GCal Events that Need To Be Updated
    (Changed on Notion but need to be changed on GCal)
    """
    # In case people deleted the Calendar Variable, this queries items where
    # the Calendar select thing is empty
    query = {
        "filter": {
            "and": [
                {"property": Calendar_Notion_Name, "select": {"is_empty": True}},
                {"property": Delete_Notion_Name, "checkbox": {"equals": False}},
            ]
        },
    }
    resultList = paginated_database_query(
        notion_client=notion, database_id=database_id, **query
    )

    if len(resultList) > 0:
        for i, el in enumerate(resultList):
            pageId = el["id"]

            # This checks off that the event has been put on Google Calendar
            notion.pages.update(
                **{
                    "page_id": pageId,
                    "properties": {
                        Calendar_Notion_Name: {
                            "select": {"name": DEFAULT_CALENDAR_NAME},
                        },
                        LastUpdatedTime_Notion_Name: {
                            "date": {
                                "start": arrow.utcnow().isoformat(),
                                "end": None,
                            }
                        },
                    },
                },
            )

    # Filter events that have been updated since the GCal event has been made

    # this query will return a dictionary that we will parse for information that
    # we want
    # look for events that are today or in the next week
    query = {
        "filter": {
            "and": [
                {
                    "property": NeedGCalUpdate_Notion_Name,
                    "checkbox": {"equals": True},
                },
                {"property": On_GCal_Notion_Name, "checkbox": {"equals": True}},
                {"property": Delete_Notion_Name, "checkbox": {"equals": False}},
            ]
        },
    }
    resultList = paginated_database_query(notion, database_id, **query)

    updatingNotionPageIds = []
    updatingCalEventIds = []

    for result in resultList:
        logging.info(result)
        logging.info("\n")
        pageId = result["id"]
        updatingNotionPageIds.append(pageId)
        logging.info("\n")
        logging.info(result)
        logging.info("\n")
        try:
            calId = result["properties"][GCalEventId_Notion_Name]["rich_text"][0][
                "text"
            ]["content"]
        except IndexError:
            calId = DEFAULT_CALENDAR_ID
        logging.info(calId)
        updatingCalEventIds.append(calId)

    TaskNames = []
    start_Dates = []
    end_Times = []
    Initiatives = []
    ExtraInfo = []
    URL_list = []
    CalendarList = []
    CurrentCalList = []

    if len(resultList) > 0:
        for i, el in enumerate(resultList):
            logging.info("\n")
            logging.info(el)
            logging.info("\n")

            TaskNames.append(
                notion_utils.collapse_rich_text_property(
                    el["properties"][Task_Notion_Name]["title"]
                )
            )
            start_Dates.append(el["properties"][Date_Notion_Name]["date"]["start"])

            if el["properties"][Date_Notion_Name]["date"]["end"] is not None:
                end_Times.append(el["properties"][Date_Notion_Name]["date"]["end"])
            else:
                end_Times.append(el["properties"][Date_Notion_Name]["date"]["start"])

            try:
                Initiatives.append(
                    get_property_text(
                        notion=notion,
                        notion_page=el,
                        property_name=Initiative_Notion_Name,
                        property_type=settings.initiative_notion_type,
                    )
                )
            except ValueError:
                Initiatives.append("")

            try:
                ExtraInfo.append(
                    el["properties"][ExtraInfo_Notion_Name]["rich_text"][0]["text"][
                        "content"
                    ]
                )
            except IndexError:
                ExtraInfo.append("")
            URL_list.append(make_task_url(el["id"], url_root))

            logging.info(el)
            # CalendarList.append(calendarDictionary[el['properties'][Calendar_Notion_Name]['select']['name']])
            try:
                CalendarList.append(
                    calendarDictionary[
                        el["properties"][Calendar_Notion_Name]["select"]["name"]
                    ]
                )
            # keyerror occurs when there's nothing put into the calendar in the first
            # place
            except KeyError:
                CalendarList.append(calendarDictionary[DEFAULT_CALENDAR_NAME])

            CurrentCalList.append(
                el["properties"][Current_Calendar_Id_Notion_Name]["rich_text"][0][
                    "text"
                ]["content"]
            )

            pageId = el["id"]

            # depending on the format of the dates, we'll update the gCal event as
            # necessary
            try:
                update_calendar_event(
                    TaskNames[i],
                    make_event_description(Initiatives[i], ExtraInfo[i]),
                    datetime.datetime.strptime(start_Dates[i], "%Y-%m-%d"),
                    URL_list[i],
                    updatingCalEventIds[i],
                    datetime.datetime.strptime(end_Times[i], "%Y-%m-%d"),
                    CurrentCalList[i],
                    CalendarList[i],
                    service,
                    settings,
                )
            except ValueError:
                try:
                    update_calendar_event(
                        TaskNames[i],
                        make_event_description(Initiatives[i], ExtraInfo[i]),
                        dateutil.parser.isoparse(start_Dates[i]),
                        URL_list[i],
                        updatingCalEventIds[i],
                        dateutil.parser.isoparse(end_Times[i]),
                        CurrentCalList[i],
                        CalendarList[i],
                        service,
                        settings,
                    )
                except ValueError:
                    update_calendar_event(
                        TaskNames[i],
                        make_event_description(Initiatives[i], ExtraInfo[i]),
                        dateutil.parser.isoparse(start_Dates[i]),
                        URL_list[i],
                        updatingCalEventIds[i],
                        dateutil.parser.isoparse(end_Times[i]),
                        CurrentCalList[i],
                        CalendarList[i],
                        service,
                        settings,
                    )

            # This updates the last time that the page in Notion was updated by the code
            notion.pages.update(
                **{
                    "page_id": pageId,
                    "properties": {
                        LastUpdatedTime_Notion_Name: {
                            "date": {
                                "start": arrow.utcnow().isoformat(),
                                "end": None,
                            }
                        },
                        Current_Calendar_Id_Notion_Name: {
                            "rich_text": [{"text": {"content": CalendarList[i]}}]
                        },
                    },
                },
            )

    else:
        logging.info("Nothing new updated to GCal")


def existing_events_gcal_to_notion(
    database_id,
    DEFAULT_CALENDAR_NAME,
    calendarDictionary,
    Date_Notion_Name,
    On_GCal_Notion_Name,
    NeedGCalUpdate_Notion_Name,
    GCalEventId_Notion_Name,
    LastUpdatedTime_Notion_Name,
    Calendar_Notion_Name,
    Current_Calendar_Id_Notion_Name,
    Delete_Notion_Name,
    service,
    notion,
    todayDate,
    settings: config.Settings,
):
    """
    Sync GCal event updates for events already in Notion back to Notion!

    Query notion tasks already in Gcal, don't have to be updated, and are today or
    in the future.
    """
    query = {
        "filter": {
            "and": [
                {
                    "property": NeedGCalUpdate_Notion_Name,
                    "formula": {"checkbox": {"equals": False}},
                },
                {"property": On_GCal_Notion_Name, "checkbox": {"equals": True}},
                # {
                #     "or": [
                #         {
                #             "property": Date_Notion_Name,
                #             "date": {"equals": todayDate},
                #         },
                #         {"property": Date_Notion_Name, "date": {"next_week": {}}},
                #     ]
                # },
                {"property": Delete_Notion_Name, "checkbox": {"equals": False}},
            ]
        },
    }

    resultList = paginated_database_query(notion, database_id, **query)

    # Comparison section:
    # We need to see what times between GCal and Notion are not the same, so we are
    # going to convert all of the notion date/times into datetime values and then
    # compare that against the datetime value of the GCal event.
    # If they are not the same, then we change the Notion event as appropriate.
    notion_IDs_List = []
    notion_start_datetimes = []
    notion_end_datetimes = []
    notion_gCal_IDs = []  # we will be comparing this against the gCal_datetimes
    gCal_start_datetimes = []
    gCal_end_datetimes = []

    notion_gCal_CalIds = (
        []
    )  # going to fill this in from the select option, not the text option.
    notion_gCal_CalNames = []
    gCal_CalIds = []

    for result in resultList:
        notion_IDs_List.append(result["id"])
        notion_start_datetimes.append(
            result["properties"][Date_Notion_Name]["date"]["start"]
        )
        notion_end_datetimes.append(
            result["properties"][Date_Notion_Name]["date"]["end"]
        )
        notion_gCal_IDs.append(
            result["properties"][GCalEventId_Notion_Name]["rich_text"][0]["text"][
                "content"
            ]
        )
        try:
            notion_gCal_CalIds.append(
                calendarDictionary[
                    result["properties"][Calendar_Notion_Name]["select"]["name"]
                ]
            )
            notion_gCal_CalNames.append(
                result["properties"][Calendar_Notion_Name]["select"]["name"]
            )
        # keyerror occurs when there's nothing put into the calendar in the first place
        except KeyError:
            notion_gCal_CalIds.append(calendarDictionary[DEFAULT_CALENDAR_NAME])
            notion_gCal_CalNames.append(
                result["properties"][Calendar_Notion_Name]["select"]["name"]
            )

    # the reason we take off the last 6 characters is so we can focus in on just the
    # date and time instead of any extra info
    for i in range(len(notion_start_datetimes)):
        try:
            notion_start_datetimes[i] = datetime.datetime.strptime(
                notion_start_datetimes[i], "%Y-%m-%d"
            )
        except ValueError:
            try:
                notion_start_datetimes[i] = datetime.datetime.strptime(
                    notion_start_datetimes[i][:-6], "%Y-%m-%dT%H:%M:%S.000"
                )
            except ValueError:
                notion_start_datetimes[i] = datetime.datetime.strptime(
                    notion_start_datetimes[i][:-6], "%Y-%m-%dT%H:%M:%S.%f"
                )

    for i in range(len(notion_end_datetimes)):
        if notion_end_datetimes[i] is not None:
            try:
                notion_end_datetimes[i] = datetime.datetime.strptime(
                    notion_end_datetimes[i], "%Y-%m-%d"
                )
            except ValueError:
                try:
                    notion_end_datetimes[i] = datetime.datetime.strptime(
                        notion_end_datetimes[i][:-6], "%Y-%m-%dT%H:%M:%S.000"
                    )
                except ValueError:
                    notion_end_datetimes[i] = datetime.datetime.strptime(
                        notion_end_datetimes[i][:-6], "%Y-%m-%dT%H:%M:%S.%f"
                    )
        else:
            # the reason we're doing this weird ass thing is because when we put the
            # end time into the update or make GCal event, it'll be representative of
            # the date
            notion_end_datetimes[i] = notion_start_datetimes[i]

    # We use the gCalId from the Notion dashboard to get retrieve the start Time from
    # the gCal event
    value = ""
    for gCalId in notion_gCal_IDs:
        # just check all of the calendars of interest for info about the event
        for calendarID in calendarDictionary.keys():
            logging.info("Trying " + calendarID + " for " + gCalId)
            try:
                x = (
                    service.events()
                    .get(calendarId=calendarDictionary[calendarID], eventId=gCalId)
                    .execute()
                )
            except HttpError:
                logging.info("Event not found")
                x = {"status": "unconfirmed"}
            if x["status"] == "confirmed":
                gCal_CalIds.append(calendarID)
                value = x
            else:
                continue

        logging.info(value)
        logging.info("\n")
        try:
            gCal_start_datetimes.append(
                dateutil.parser.isoparse(value["start"]["dateTime"])  # type: ignore
            )
        except KeyError:
            date = datetime.datetime.strptime(value["start"]["date"], "%Y-%m-%d")  # type: ignore # noqa
            # x = datetime(date.year, date.month, date.day, 0, 0, 0) redundant I think
            # gCal_start_datetimes.append(datetime.strptime(x, "%Y-%m-%dT%H:%M:%S"))
            gCal_start_datetimes.append(date)
        try:
            gCal_end_datetimes.append(
                dateutil.parser.isoparse(value["end"]["dateTime"])  # type: ignore
            )
        except KeyError:
            date = datetime.datetime.strptime(value["end"]["date"], "%Y-%m-%d")  # type: ignore # noqa
            x = datetime.datetime(
                date.year, date.month, date.day, 0, 0, 0
            ) - datetime.timedelta(days=1)
            gCal_end_datetimes.append(x)

    # Now we iterate and compare the time on the Notion Dashboard and the start time of
    # the GCal event
    # If the datetimes don't match up,  then the Notion  Dashboard must be updated

    new_notion_start_datetimes: list[None | datetime.datetime] = []
    new_notion_end_datetimes: list[None | datetime.datetime] = []

    for i in range(len(notion_start_datetimes)):
        if notion_start_datetimes[i] != gCal_start_datetimes[i]:
            new_notion_start_datetimes.append(gCal_start_datetimes[i])
        else:
            new_notion_start_datetimes.append(None)

        if notion_end_datetimes[i] != gCal_end_datetimes[i]:
            # this means that there is no end time in notion
            new_notion_end_datetimes.append(gCal_end_datetimes[i])
        else:
            new_notion_end_datetimes.append(None)

    logging.info("test")
    logging.info(new_notion_start_datetimes)
    logging.info(new_notion_end_datetimes)
    logging.info("\n")
    for i in range(len(notion_gCal_IDs)):
        logging.info(
            notion_start_datetimes[i], gCal_start_datetimes[i], notion_gCal_IDs[i]
        )

    for i, (new_start, new_end) in enumerate(
        zip(new_notion_start_datetimes, new_notion_end_datetimes)
    ):
        if (
            new_start is not None and new_end is not None
        ):  # both start and end time need to be updated
            start: datetime.datetime = new_start
            end: datetime.datetime = new_end

            # you're given 12 am dateTimes so you want to enter them as dates (not
            # datetimes) into Notion
            if start.hour == 0 and start.minute == 0 and start == end:
                # update the notion dashboard with the new datetime and update the last
                # updated time
                notion.pages.update(
                    **{
                        "page_id": notion_IDs_List[i],
                        "properties": {
                            Date_Notion_Name: {
                                "date": {
                                    "start": start.strftime("%Y-%m-%d"),
                                    "end": None,
                                }
                            },
                            LastUpdatedTime_Notion_Name: {
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                }
                            },
                        },
                    },
                )
            elif (
                start.hour == 0
                and start.minute == 0
                and end.hour == 0
                and end.minute == 0
            ):
                # you're given 12 am dateTimes so you want to enter them as dates (not
                # datetimes) into Notion
                # update the notion dashboard with the new datetime and update the last
                # updated time
                notion.pages.update(
                    **{
                        "page_id": notion_IDs_List[i],
                        "properties": {
                            Date_Notion_Name: {
                                "date": {
                                    "start": start.strftime("%Y-%m-%d"),
                                    "end": end.strftime("%Y-%m-%d"),
                                }
                            },
                            LastUpdatedTime_Notion_Name: {
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                }
                            },
                        },
                    },
                )
            else:  # update Notion using datetime format
                notion.pages.update(
                    # update the notion dashboard with the new datetime and update the
                    # last updated time
                    **{
                        "page_id": notion_IDs_List[i],
                        "properties": {
                            Date_Notion_Name: {
                                "date": {
                                    "start": start.isoformat(),
                                    "end": end.isoformat(),
                                }
                            },
                            LastUpdatedTime_Notion_Name: {
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                }
                            },
                        },
                    },
                )
        elif new_start is not None:  # only start time need to be updated
            start = new_start
            end = notion_end_datetimes[i]

            if start.hour == 0 and start.minute == 0 and start == end:
                # you're given 12 am dateTimes so you want to enter them as dates (not
                # datetimes) into Notion
                # update the notion dashboard with the new datetime and update the last
                # updated time
                notion.pages.update(
                    **{
                        "page_id": notion_IDs_List[i],
                        "properties": {
                            Date_Notion_Name: {
                                "date": {
                                    "start": start.strftime("%Y-%m-%d"),
                                    "end": None,
                                }
                            },
                            LastUpdatedTime_Notion_Name: {
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                }
                            },
                        },
                    },
                )
            elif (
                start.hour == 0
                and start.minute == 0
                and end.hour == 0
                and end.minute == 0
            ):
                # you're given 12 am dateTimes so you want to enter them as dates
                # (not datetimes) into Notion
                notion.pages.update(
                    # update the notion dashboard with the new datetime and update the
                    # last updated time
                    **{
                        "page_id": notion_IDs_List[i],
                        "properties": {
                            Date_Notion_Name: {
                                "date": {
                                    "start": start.strftime("%Y-%m-%d"),
                                    "end": end.strftime("%Y-%m-%d"),
                                }
                            },
                            LastUpdatedTime_Notion_Name: {
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                }
                            },
                        },
                    },
                )
            else:
                # update Notion using datetime format
                notion.pages.update(
                    # update the notion dashboard with the new datetime and update the
                    # last updated time
                    **{
                        "page_id": notion_IDs_List[i],
                        "properties": {
                            Date_Notion_Name: {
                                "date": {
                                    "start": start.isoformat(),
                                    "end": end.isoformat(),
                                }
                            },
                            LastUpdatedTime_Notion_Name: {
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                }
                            },
                        },
                    },
                )
        elif new_end is not None:  # only end time needs to be updated
            start = notion_start_datetimes[i]
            end = new_end

            if start.hour == 0 and start.minute == 0 and start == end:
                # you're given 12 am dateTimes so you want to enter them as dates (not
                # datetimes) into Notion
                notion.pages.update(
                    # update the notion dashboard with the new datetime and update the
                    # last updated time
                    **{
                        "page_id": notion_IDs_List[i],
                        "properties": {
                            Date_Notion_Name: {
                                "date": {
                                    "start": start.strftime("%Y-%m-%d"),
                                    "end": None,
                                }
                            },
                            LastUpdatedTime_Notion_Name: {
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                }
                            },
                        },
                    },
                )
            elif (
                start.hour == 0
                and start.minute == 0
                and end.hour == 0
                and end.minute == 0
            ):
                # update the notion dashboard with the new datetime and update the last
                # updated time
                notion.pages.update(
                    **{
                        "page_id": notion_IDs_List[i],
                        "properties": {
                            Date_Notion_Name: {
                                "date": {
                                    "start": start.strftime("%Y-%m-%d"),
                                    "end": end.strftime("%Y-%m-%d"),
                                }
                            },
                            LastUpdatedTime_Notion_Name: {
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                }
                            },
                        },
                    },
                )
            else:
                # update Notion using datetime format
                notion.pages.update(
                    # update the notion dashboard with the new datetime and update the
                    # last updated time
                    **{
                        "page_id": notion_IDs_List[i],
                        "properties": {
                            Date_Notion_Name: {
                                "date": {
                                    "start": start.isoformat(),
                                    "end": end.isoformat(),
                                }
                            },
                            LastUpdatedTime_Notion_Name: {
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                }
                            },
                        },
                    },
                )
        else:  # nothing needs to be updated here
            continue

    logging.info(notion_IDs_List)
    logging.info("\n")
    logging.info(gCal_CalIds)

    CalNames = list(calendarDictionary.keys())
    CalIds = list(calendarDictionary.values())

    for i, gCalId in enumerate(gCal_CalIds):
        # instead of checking, just update the notion datebase with whatever calendar
        # the event is on
        logging.info("GcalId: " + gCalId)
        notion.pages.update(
            # This puts the the GCal Id into the Notion Dashboard
            **{
                "page_id": notion_IDs_List[i],
                "properties": {
                    Current_Calendar_Id_Notion_Name: {  # this is the text
                        "rich_text": [
                            {"text": {"content": CalIds[CalNames.index(gCalId)]}}
                        ]
                    },
                    Calendar_Notion_Name: {  # this is the select
                        "select": {"name": gCalId},
                    },
                    LastUpdatedTime_Notion_Name: {
                        "date": {
                            "start": arrow.utcnow().isoformat(),
                            "end": None,
                        }
                    },
                },
            },
        )


def new_events_gcal_to_notion(
    database_id,
    calendarDictionary,
    Task_Notion_Name,
    Date_Notion_Name,
    ExtraInfo_Notion_Name,
    On_GCal_Notion_Name,
    GCalEventId_Notion_Name,
    LastUpdatedTime_Notion_Name,
    Calendar_Notion_Name,
    Current_Calendar_Id_Notion_Name,
    Delete_Notion_Name,
    service,
    notion,
    settings: config.Settings,
):
    """
    Bring events (not in Notion already) from GCal to Notion

    First, we get a list of all of the GCal Event Ids from the Notion Dashboard.
    """
    my_page = paginated_database_query(
        notion,
        database_id,
        **{
            "filter": {
                "and": [
                    {
                        "property": GCalEventId_Notion_Name,
                        "text": {"is_not_empty": True},
                    },
                    {"property": Delete_Notion_Name, "checkbox": {"equals": False}},
                ]
            },
        },
    )

    my_page = paginated_database_query(
        notion,
        database_id,
        **{
            "filter": {
                "property": GCalEventId_Notion_Name,
                "text": {"is_not_empty": True},
            },
        },
    )

    resultList = my_page

    ALL_notion_gCal_Ids = []

    for result in resultList:
        ALL_notion_gCal_Ids.append(
            result["properties"][GCalEventId_Notion_Name]["rich_text"][0]["text"][
                "content"
            ]
        )

    # Get the GCal Ids and other Event Info from Google Calendar

    events = []
    # get all the events from all calendars of interest
    for key, value in calendarDictionary.items():
        x = (
            service.events()
            .list(calendarId=value, maxResults=2000, timeMin=arrow.utcnow().isoformat())
            .execute()
        )
        events.extend(x["items"])
        time.sleep(0.1)

    logging.info(events)

    calItems = events

    calName = [item["summary"] for item in calItems]

    gCal_calendarId = [
        item["organizer"]["email"] for item in calItems
    ]  # this is to get all of the calendarIds for each event

    CalNames = list(calendarDictionary.keys())
    CalIds = list(calendarDictionary.values())
    gCal_calendarName = [CalNames[CalIds.index(x)] for x in gCal_calendarId]

    calStartDates = []
    calEndDates = []
    for el in calItems:
        try:
            calStartDates.append(dateutil.parser.isoparse(el["start"]["dateTime"]))
        except KeyError:
            date = datetime.datetime.strptime(el["start"]["date"], "%Y-%m-%d")
            x = datetime.datetime(date.year, date.month, date.day, 0, 0, 0)
            calStartDates.append(x)
        try:
            calEndDates.append(dateutil.parser.isoparse(el["end"]["dateTime"]))
        except KeyError:
            date = datetime.datetime.strptime(el["end"]["date"], "%Y-%m-%d")
            x = datetime.datetime(date.year, date.month, date.day, 0, 0, 0)
            calEndDates.append(x)

    calIds = [item["id"] for item in calItems]
    calDescriptions = []
    for item in calItems:
        try:
            calDescriptions.append(item["description"])
        except KeyError:
            calDescriptions.append(" ")

    # Now, we compare the Ids from Notion and Ids from GCal. If the Id from GCal is
    # not in the list from Notion, then we know that the event does not exist in
    # Notion yet, so we should bring that over.

    for i in range(len(calIds)):
        if calIds[i] not in ALL_notion_gCal_Ids:
            if calStartDates[i] == calEndDates[i] - datetime.timedelta(
                days=1
            ):  # only add in the start DATE
                # Here, we create a new page for every new GCal event
                end = calEndDates[i] - datetime.timedelta(days=1)
                my_page = notion.pages.create(
                    **{
                        "parent": {
                            "database_id": database_id,
                        },
                        "properties": {
                            Task_Notion_Name: {
                                "type": "title",
                                "title": [
                                    {
                                        "type": "text",
                                        "text": {
                                            "content": calName[i],
                                        },
                                    },
                                ],
                            },
                            Date_Notion_Name: {
                                "type": "date",
                                "date": {
                                    "start": calStartDates[i].strftime("%Y-%m-%d"),
                                    "end": None,
                                },
                            },
                            LastUpdatedTime_Notion_Name: {
                                "type": "date",
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                },
                            },
                            ExtraInfo_Notion_Name: {
                                "type": "rich_text",
                                "rich_text": [
                                    {"text": {"content": calDescriptions[i]}}
                                ],
                            },
                            GCalEventId_Notion_Name: {
                                "type": "rich_text",
                                "rich_text": [{"text": {"content": calIds[i]}}],
                            },
                            On_GCal_Notion_Name: {"type": "checkbox", "checkbox": True},
                            Current_Calendar_Id_Notion_Name: {
                                "rich_text": [{"text": {"content": gCal_calendarId[i]}}]
                            },
                            Calendar_Notion_Name: {
                                "select": {"name": gCal_calendarName[i]},
                            },
                        },
                    },
                )

            elif (
                calStartDates[i].hour == 0
                and calStartDates[i].minute == 0
                and calEndDates[i].hour == 0
                and calEndDates[i].minute == 0
            ):  # add start and end in DATE format
                # Here, we create a new page for every new GCal event
                end = calEndDates[i] - datetime.timedelta(days=1)

                my_page = notion.pages.create(
                    **{
                        "parent": {
                            "database_id": database_id,
                        },
                        "properties": {
                            Task_Notion_Name: {
                                "type": "title",
                                "title": [
                                    {
                                        "type": "text",
                                        "text": {
                                            "content": calName[i],
                                        },
                                    },
                                ],
                            },
                            Date_Notion_Name: {
                                "type": "date",
                                "date": {
                                    "start": calStartDates[i].strftime("%Y-%m-%d"),
                                    "end": end.strftime("%Y-%m-%d"),
                                },
                            },
                            LastUpdatedTime_Notion_Name: {
                                "type": "date",
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                },
                            },
                            ExtraInfo_Notion_Name: {
                                "type": "rich_text",
                                "rich_text": [
                                    {"text": {"content": calDescriptions[i]}}
                                ],
                            },
                            GCalEventId_Notion_Name: {
                                "type": "rich_text",
                                "rich_text": [{"text": {"content": calIds[i]}}],
                            },
                            On_GCal_Notion_Name: {"type": "checkbox", "checkbox": True},
                            Current_Calendar_Id_Notion_Name: {
                                "rich_text": [{"text": {"content": gCal_calendarId[i]}}]
                            },
                            Calendar_Notion_Name: {
                                "select": {"name": gCal_calendarName[i]},
                            },
                        },
                    },
                )

            else:  # regular datetime stuff
                # Here, we create a new page for every new GCal event
                my_page = notion.pages.create(
                    **{
                        "parent": {
                            "database_id": database_id,
                        },
                        "properties": {
                            Task_Notion_Name: {
                                "type": "title",
                                "title": [
                                    {
                                        "type": "text",
                                        "text": {
                                            "content": calName[i],
                                        },
                                    },
                                ],
                            },
                            Date_Notion_Name: {
                                "type": "date",
                                "date": {
                                    "start": calStartDates[i].isoformat(),
                                    "end": calEndDates[i].isoformat(),
                                },
                            },
                            LastUpdatedTime_Notion_Name: {
                                "type": "date",
                                "date": {
                                    "start": arrow.utcnow().isoformat(),
                                    "end": None,
                                },
                            },
                            ExtraInfo_Notion_Name: {
                                "type": "rich_text",
                                "rich_text": [
                                    {"text": {"content": calDescriptions[i]}}
                                ],
                            },
                            GCalEventId_Notion_Name: {
                                "type": "rich_text",
                                "rich_text": [{"text": {"content": calIds[i]}}],
                            },
                            On_GCal_Notion_Name: {"type": "checkbox", "checkbox": True},
                            Current_Calendar_Id_Notion_Name: {
                                "rich_text": [{"text": {"content": gCal_calendarId[i]}}]
                            },
                            Calendar_Notion_Name: {
                                "select": {"name": gCal_calendarName[i]},
                            },
                        },
                    },
                )

            logging.info(f"Added this event to Notion: {calName[i]}")


def delete_done_pages(
    notion: nc.Client,
    database_id: str,
    GCalEventId_Notion_Name,
    On_GCal_Notion_Name,
    Delete_Notion_Name,
    DELETE_OPTION: bool,
    calendarDictionary,
    Calendar_Notion_Name,
    service,
):
    """Deletion Sync

    - If marked *Done* in Notion, then it will delete the GCal event
    (and the Notion event once Python API updates)
    """
    resultList = paginated_database_query(
        notion,
        database_id,
        **{
            "filter": {
                "and": [
                    {
                        "property": GCalEventId_Notion_Name,
                        "text": {"is_not_empty": True},
                    },
                    {"property": On_GCal_Notion_Name, "checkbox": {"equals": True}},
                    {"property": Delete_Notion_Name, "checkbox": {"equals": True}},
                ]
            },
        },
    )

    # delete gCal event (and Notion task once the Python API is updated)
    if DELETE_OPTION and len(resultList) > 0:
        for i, el in enumerate(resultList):
            calendarID = calendarDictionary[
                el["properties"][Calendar_Notion_Name]["select"]["name"]
            ]
            eventId = el["properties"][GCalEventId_Notion_Name]["rich_text"][0]["text"][
                "content"
            ]

            try:
                service.events().delete(
                    calendarId=calendarID, eventId=eventId
                ).execute()
                logging.info("deleted:", calendarID, eventId)
            except HttpError:
                continue
            time.sleep(0.1)


def make_event_description(initiative, info):
    """Method to make a calendar event description

    This method can be edited as wanted. Whatever is returned from this method will
    be in the GCal event description
    Whatever you change up, be sure to return a string
    """
    if initiative == "" and info == "":
        return ""
    elif info == "":
        return initiative
    elif initiative == "":
        return info
    else:
        return f"Initiative: {initiative} \n{info}"


def make_task_url(ending: str, url_root: str):
    """Method to create a task's url"""
    # To make a url for the notion task, we have to take the id of the task and take
    # away the hyphens from the string
    url_id = ending.replace("-", "")
    return url_root + url_id


def make_cal_event(
    eventName,
    eventDescription,
    eventStartTime,
    sourceURL,
    eventEndTime,
    calId,
    service,
    config: config.Settings,
):
    """Method to make a calendar event"""
    if (
        eventStartTime.hour == 0
        and eventStartTime.minute == 0
        and eventEndTime == eventStartTime
    ):  # only startTime is given from the Notion Dashboard
        if config.all_day_event_option == 1:
            eventStartTime = datetime.datetime.combine(
                eventStartTime, datetime.datetime.min.time()
            ) + datetime.timedelta(
                hours=config.default_event_start
            )  # make the events pop up at 8 am instead of 12 am
            eventEndTime = eventStartTime + datetime.timedelta(
                minutes=config.default_event_length
            )
            event = {
                "summary": eventName,
                "description": eventDescription,
                "start": {
                    "dateTime": eventStartTime.strftime("%Y-%m-%dT%H:%M:%S"),
                    "timeZone": config.timezone,
                },
                "end": {
                    "dateTime": eventEndTime.strftime("%Y-%m-%dT%H:%M:%S"),
                    "timeZone": config.timezone,
                },
                "source": {
                    "title": "Notion Link",
                    "url": sourceURL,
                },
            }
        else:
            eventEndTime = eventEndTime + datetime.timedelta(
                days=1
            )  # gotta make it to 12AM the day after
            event = {
                "summary": eventName,
                "description": eventDescription,
                "start": {
                    "date": eventStartTime.strftime("%Y-%m-%d"),
                    "timeZone": config.timezone,
                },
                "end": {
                    "date": eventEndTime.strftime("%Y-%m-%d"),
                    "timeZone": config.timezone,
                },
                "source": {
                    "title": "Notion Link",
                    "url": sourceURL,
                },
            }
    elif (
        eventStartTime.hour == 0
        and eventStartTime.minute == 0
        and eventEndTime.hour == 0
        and eventEndTime.minute == 0
        and eventStartTime != eventEndTime
    ):

        eventEndTime = eventEndTime + datetime.timedelta(
            days=1
        )  # gotta make it to 12AM the day after

        event = {
            "summary": eventName,
            "description": eventDescription,
            "start": {
                "date": eventStartTime.strftime("%Y-%m-%d"),
                "timeZone": config.timezone,
            },
            "end": {
                "date": eventEndTime.strftime("%Y-%m-%d"),
                "timeZone": config.timezone,
            },
            "source": {
                "title": "Notion Link",
                "url": sourceURL,
            },
        }

    else:
        if eventStartTime.hour == 0 and eventStartTime.minute == 0:
            if eventEndTime == eventStartTime:
                # if the datetime fed into this is only a date or is at 12 AM,
                # then the event will fall under here
                eventStartTime = datetime.datetime.combine(
                    eventStartTime, datetime.datetime.min.time()
                ) + datetime.timedelta(
                    hours=config.default_event_start
                )  # make the events pop up at 8 am instead of 12 am
                eventEndTime = eventStartTime + datetime.timedelta(
                    minutes=config.default_event_length
                )
        elif eventEndTime == eventStartTime:
            # this would meant that only 1 datetime was actually on the notion dashboard
            eventEndTime = eventStartTime + datetime.timedelta(
                minutes=config.default_event_length
            )

        event = {
            "summary": eventName,
            "description": eventDescription,
            "start": {
                "dateTime": eventStartTime.strftime("%Y-%m-%dT%H:%M:%S"),
                "timeZone": config.timezone,
            },
            "end": {
                "dateTime": eventEndTime.strftime("%Y-%m-%dT%H:%M:%S"),
                "timeZone": config.timezone,
            },
            "source": {
                "title": "Notion Link",
                "url": sourceURL,
            },
        }
    logging.info(f"Adding this event to calendar: {eventName}")

    logging.info(event)
    x = service.events().insert(calendarId=calId, body=event).execute()
    return x["id"]


def update_calendar_event(
    eventName,
    eventDescription,
    eventStartTime,
    sourceURL,
    eventId,
    eventEndTime,
    currentCalId,
    CalId,
    service,
    config: config.Settings,
):
    """Update a Google Calendar event.

    Args:
        eventName:
        eventDescription:
        eventStartTime:
        sourceURL:
        eventId:
        eventEndTime:
        currentCalId:
        CalId:
        service:
        config (config.Settings):

    Returns:
        _type_: _description_
    """
    if (
        eventStartTime.hour == 0
        and eventStartTime.minute == 0
        and eventEndTime == eventStartTime
    ):  # you're given a single date
        if config.all_day_event_option == 1:
            eventStartTime = datetime.datetime.combine(
                eventStartTime, datetime.datetime.min.time()
            ) + datetime.timedelta(
                hours=config.default_event_start
            )  # make the events pop up at 8 am instead of 12 am
            eventEndTime = eventStartTime + datetime.timedelta(
                minutes=config.default_event_length
            )
            event = {
                "summary": eventName,
                "description": eventDescription,
                "start": {
                    "dateTime": eventStartTime.strftime("%Y-%m-%dT%H:%M:%S"),
                    "timeZone": config.timezone,
                },
                "end": {
                    "dateTime": eventEndTime.strftime("%Y-%m-%dT%H:%M:%S"),
                    "timeZone": config.timezone,
                },
                "source": {
                    "title": "Notion Link",
                    "url": sourceURL,
                },
            }
        else:
            eventEndTime = eventEndTime + datetime.timedelta(
                days=1
            )  # gotta make it to 12AM the day after
            event = {
                "summary": eventName,
                "description": eventDescription,
                "start": {
                    "date": eventStartTime.strftime("%Y-%m-%d"),
                    "timeZone": config.timezone,
                },
                "end": {
                    "date": eventEndTime.strftime("%Y-%m-%d"),
                    "timeZone": config.timezone,
                },
                "source": {
                    "title": "Notion Link",
                    "url": sourceURL,
                },
            }
    elif (
        eventStartTime.hour == 0
        and eventStartTime.minute == 0
        and eventEndTime.hour == 0
        and eventEndTime.minute == 0
        and eventStartTime != eventEndTime
    ):  # it's a multiple day event

        eventEndTime = eventEndTime + datetime.timedelta(
            days=1
        )  # gotta make it to 12AM the day after

        event = {
            "summary": eventName,
            "description": eventDescription,
            "start": {
                "date": eventStartTime.strftime("%Y-%m-%d"),
                "timeZone": config.timezone,
            },
            "end": {
                "date": eventEndTime.strftime("%Y-%m-%d"),
                "timeZone": config.timezone,
            },
            "source": {
                "title": "Notion Link",
                "url": sourceURL,
            },
        }

    else:  # just 2 datetimes passed in
        if eventStartTime.hour == 0 and eventStartTime.minute == 0:
            if eventEndTime != eventStartTime:
                # Start on Notion is 12 am and end is also given on Notion
                pass
            else:
                # if the datetime fed into this is only a date or is at 12 AM,
                # then the event will fall under here
                eventStartTime = datetime.datetime.combine(
                    eventStartTime, datetime.datetime.min.time()
                ) + datetime.timedelta(
                    hours=config.default_event_start
                )  # make the events pop up at 8 am instead of 12 am
                eventEndTime = eventStartTime + datetime.timedelta(
                    minutes=config.default_event_length
                )
        elif eventEndTime == eventStartTime:
            # this would meant that only 1 datetime was actually on the notion dashboard
            eventEndTime = eventStartTime + datetime.timedelta(
                minutes=config.default_event_length
            )

        event = {
            "summary": eventName,
            "description": eventDescription,
            "start": {
                "dateTime": eventStartTime.strftime("%Y-%m-%dT%H:%M:%S"),
                "timeZone": config.timezone,
            },
            "end": {
                "dateTime": eventEndTime.strftime("%Y-%m-%dT%H:%M:%S"),
                "timeZone": config.timezone,
            },
            "source": {
                "title": "Notion Link",
                "url": sourceURL,
            },
        }
    logging.info(f"Updating this event to calendar: {eventName}")

    if currentCalId == CalId:
        x = (
            service.events()
            .update(calendarId=CalId, eventId=eventId, body=event)
            .execute()
        )

    else:
        # When we have to move the event to a new calendar.
        # We must move the event over to the new calendar and
        # then update the information on the event
        logging.info("Event " + eventId)
        logging.info("CurrentCal " + currentCalId)
        logging.info("NewCal " + CalId)
        x = (
            service.events()
            .move(calendarId=currentCalId, eventId=eventId, destination=CalId)
            .execute()
        )
        logging.info("New event id: " + x["id"])
        x = (
            service.events()
            .update(calendarId=CalId, eventId=eventId, body=event)
            .execute()
        )

    return x["id"]
